#include<iostream>
#include<fstream>

using namespace std;

class Modell
{
    protected:
        int anzahl;
    public:
        Modell(int anzahl)
        {
            this->anzahl=anzahl;
        }


        virtual double rechteSeite(double* y, int zeile)=0;
        
        int holeAnzahlGleichungen()
        {
            return anzahl;
        } 
}; 


class ModellSchwinger : public Modell
{
    public:
        ModellSchwinger() : Modell(2)  
        {
        }
        double rechteSeite(double* y, int zeile)
        {
            switch(zeile)
            {
                case 0:
                     return y[1];
                break;
                case 1:
//                     return -y[0]-0.5*y[1]; 
                     return -y[0]; 
                break;                    
                default:
                     return 0.0;
                break;                                                                                            
            }               
        }
}; 

class ModellTilger : public Modell
{
    public:
        ModellTilger() : Modell(4)  
        {
            m1 = 3.0;
            C1 = 27.0;
            m2 = 1.0;
            C2 = 13.0;
            D2 = 4.0;                          
        }
        double rechteSeite(double* y, int zeile)
        {
            switch(zeile)
            {
                case 0:
                     return y[1];
                break;
                case 1:
                     return (-C1/m1-C2/m1)*y[0] + (-D2/m1)*y[1] + (C2/m1)*y[2] + (D2/m1)*y[3]; 
                break;                    
                case 2:
                     return y[3]; 
                break;                    
                case 3:
                     return (C2/m2)*y[0] + (D2/m2)*y[1] + (-C2/m2)*y[2] + (-D2/m2)*y[3]; 
                break;                    
                default:
                     return 0.0;
                break;                                                                                            
            }               
        }
        void setParameter(double C2, double D2)
        {
            this->C2 = C2;
            this->D2 = D2;             
        }
    private:
        double m1,C1,m2,C2,D2;        
}; 


class Integrator
{
    public:
        Integrator(Modell* modell)
        {
            this->modell = modell;
            yneu = new double[modell->holeAnzahlGleichungen()];
        }
        virtual double* zeitschritt(double* yalt,double dt) = 0;
    protected:
        Modell* modell;
        double* yneu;
}; 


class IntegratorRuKu : public Integrator
{
    public:
        IntegratorRuKu(Modell* modell) : Integrator(modell)
        {
            yhilf = new double[modell->holeAnzahlGleichungen()];
            k1 = new double[modell->holeAnzahlGleichungen()]; 
            k2 = new double[modell->holeAnzahlGleichungen()]; 
            k3 = new double[modell->holeAnzahlGleichungen()]; 
            k4 = new double[modell->holeAnzahlGleichungen()]; 
        }
        double* zeitschritt(double* yalt,double dt)
        {
            for(int i=0;i<modell->holeAnzahlGleichungen();i++)
            {
                k1[i] = modell->rechteSeite(yalt,i);
                yhilf[i] = yalt[i] + 0.5*dt*k1[i];
            }                
            for(int i=0;i<modell->holeAnzahlGleichungen();i++)
            {
                k2[i] = modell->rechteSeite(yhilf,i);
                yhilf[i] = yalt[i] + 0.5*dt*k2[i];
            }                
            for(int i=0;i<modell->holeAnzahlGleichungen();i++)
            {
                k3[i] = modell->rechteSeite(yhilf,i);
                yhilf[i] = yalt[i] + dt*k3[i];
            }                
            for(int i=0;i<modell->holeAnzahlGleichungen();i++)
            {
                k4[i] = modell->rechteSeite(yhilf,i);
            }                

            for(int i=0;i<modell->holeAnzahlGleichungen();i++)
            {
                yneu[i] = yalt[i] + (dt/6.0)*(k1[i]+2.0*k2[i]+2.0*k3[i]+k4[i]);
            }                

            return yneu;
        } 
    private:
        double* yhilf;
        double *k1,*k2,*k3,*k4;
}; 

class IntegratorEuler : public Integrator
{
    public:
        IntegratorEuler(Modell* modell) : Integrator(modell)
        {
        }
        double* zeitschritt(double* yalt,double dt)
        {
            for(int i=0;i<modell->holeAnzahlGleichungen();i++)
            {
                f = modell->rechteSeite(yalt,i);
                yneu[i] = yalt[i] + f*dt;
            }
            return yneu;
        }
    private:
        double f;                     
}; 

void simulieren(Integrator* integrator,string dateiname)
{
    double t=0.0;
    double dt=0.005;

    double* yneu;
    double yalt[4] = {0.5,0.0,0.0,0.0};

    ofstream datei(dateiname.data());

    for(int i=0;i<2001;i++)
    {
        datei<<t<<" "<<yalt[0]<<endl;

        yneu = integrator->zeitschritt(yalt,dt);

        yalt[0] = yneu[0];
        yalt[1] = yneu[1];
        yalt[2] = yneu[2];
        yalt[3] = yneu[3];

        t=t+dt;
    }

    datei.close();
}

int main()
{
    ModellTilger*     modell          = new ModellTilger();
    IntegratorRuKu*      integratorRuKu  = new IntegratorRuKu(modell);

    simulieren(integratorRuKu,"test1.txt");
}
